/*!
 * @file new_class_id.h
 * @brief Definition of a dynamically allocating ClassID
 *
 */

#ifndef _NEW_CLASS_ID_H
#define _NEW_CLASS_ID_H

#include <string>

class NewObjectListBase;

//! A class to dynamically allocate ClassID's and support a isA operator
class NewClassID
{
public:
  NewClassID();
  NewClassID(const NewObjectListBase* id);
  // the copy constructor is also defined.
  const int& id() const { return *_id; };
  const std::string& name() const { return *_name; };

  bool operator==(const NewClassID& id) const { return *_id == *id._id /* || _name == id._name */; };
  bool operator==(int id) const { return *_id == id; };
  bool operator==(const std::string& name) const { return *_name == name; };
  bool operator!=(const NewClassID& id) const { return *_id != *id._id /* && _name != id._name*/;  };
  bool operator!=(int id) const { return *_id != id; };
  bool operator!=(const std::string& name) const { return *_name != name; };

private:
  const int*                _id;
  const std::string*        _name;
};

class NullClass
{
public:
  static NewClassID classID() { return NullClass::_classID; }
private:
  NullClass();
private:
  static NewClassID _classID;
};

#endif /* _NEW_CLASS_ID_H */
